package PublishDraft::App::CMS;

use strict;
use warnings;

use Exporter;
@PublishDraft::App::CMS::ISA = qw( Exporter );
use vars qw( @EXPORT_OK );
@EXPORT_OK = qw( rest_day is_limit );

use Encode;
use CustomFields::Field;

use MT::Util qw( is_valid_email is_url dirify offset_time_list format_ts );
use MT::I18N qw( wrap_text );
use CustomFields::Util qw( get_meta save_meta );
################################################
use PublishDraft::Util qw( get_lastest_revision get_lastest_revision_object get_lastest_revision_for_future_post get_lastest_revision_object_for_future_post );
################################################

use Data::Dumper;

sub pre_save {
    my ($eh, $app, $obj, $orig_obj) = @_;

    return $app->error( $app->translate("expiration of a term.") )
        if is_limit();

    1;
}

sub post_save {
    my ($eh, $app, $obj, $orig_obj) = @_;

    my $status = $obj->status;
    if ($status == 2) {

        my $revision = get_lastest_revision($app);
        return 1 if !$revision;

        $obj->current_revision($revision->rev_number) if $revision;

        my $rev = $obj->load_revision( { rev_number => $revision->rev_number } );
        if ( $rev && @$rev ) {
            $status = 30 if $rev->[0]->status == 20;
        }
    }
}

sub save_snapshot_wf {
    my $app = shift;

    return $app->error( $app->translate("expiration of a term.") )
        if is_limit();

    my $q     = $app->param;
    my $type  = $q->param('_type');
    my $id    = $q->param('id');
    my $blog_id    = $q->param('blog_id');
    my $param = {};

    return $app->errtrans("Invalid request.")
        unless $type;

    $app->validate_magic() or return;

    my $class = $app->model($type)
        or return $app->errtrans( "Invalid type [_1]", $type );

    return $app->error( $app->translate( "No ID" ) ) if !$id;

    my $revision = get_lastest_revision($app);
    return $app->error( "No Revision." ) if !$revision;

    my $obj = get_lastest_revision_object($app, $revision);
    return $app->error( $app->translate( "Invalid type [_1]", $type ) ) if $obj->class ne $type;

    my $original = $obj->clone();
    $original->{__tags} = $obj->{__tags};

    my @category_list;
    my $categories = $obj->cache_property( 'categories' );
    foreach my $c (@{$categories}) {
        push @category_list, $c->id;
    }
    $original->{__cats} = \@category_list;

    my $names    = $obj->column_names;
    my %values   = map { $_ => ( scalar $q->param($_) ) } @$names;

    if ( ( 'entry' eq $type ) || ( 'page' eq $type ) ) {
        $app->_translate_naughty_words($obj);
    }
    delete $values{'id'} if exists( $values{'id'} ) && !$values{'id'};
    $obj->set_values( \%values );

######################
    my $date = $q->param('authored_on_date');
    my $time = $q->param('authored_on_time');
    my $ts   = $date . $time;
    $ts =~ s/\D//g;
    $obj->authored_on($ts);
######################

    $obj->current_revision($revision->rev_number);

#    $obj->status(10) if $obj->status == 40;

    my @obj_assets = MT::ObjectAsset->load(
                              {
                                  object_ds => $obj->class,
                                  object_id => $id
                              } );
    my @orig_asset_ids = map { $_->asset_id } @obj_assets;
    $obj->cache_property( 'orig_object_assets', undef, \@orig_asset_ids );

    my $include_asset_ids = $app->param('include_asset_ids');
    my @asset_ids = split( ',', $include_asset_ids );
    $obj->cache_property( 'object_assets', undef, \@asset_ids );

    my $cat;
    my $cat_ids = $q->param('category_ids');
    if ($cat_ids) {
        my @cats = split /,/, $cat_ids;
        if (@cats) {
            my $primary_cat = $cats[0];
            $cat = MT::Category->load(
                { id => $primary_cat, blog_id => $blog_id } );
            my @categories
                = MT::Category->load( { id => \@cats, blog_id => $blog_id } );
            $obj->cache_property( 'category',   undef, $cat );
            $obj->cache_property( 'categories', undef, \@categories );
        }
    }
    else {
        $obj->cache_property( 'category', undef, undef );
        $obj->cache_property( 'categories', undef, [] );
    }

    if ($q->param('tags')) {
        my @tag_names = split(/\s*,\s*/,$q->param('tags'));
        my @save_tags;
        foreach my $tag_name (@tag_names) {
            next if $tag_name =~ m/^@/;

            my $tag;
            my @tags = MT::Tag->load({name => $tag_name});
            if (!@tags) {
                $tag = MT::Tag->new;
                $tag->name($tag_name);
                $tag->save;
            } else {
               $tag = shift @tags;
            }
            push(@save_tags, $tag);
        }
        $obj->{__tag_objects} = \@save_tags;
    } else {
        delete $obj->{__tags};
        delete $obj->{__tag_objects};
    }

    _set_customfield_objs($app, $obj);

    _gather_changed_cols( $obj, $original, $app );

    if ($obj->{changed_revisioned_cols}) {
        my $col = 'max_revisions_' . $obj->datasource;
        if ( my $blog = $obj->blog ) {
            my $max = $blog->$col;
            $obj->handle_max_revisions($max);
        }

        my $revision = $obj->save_revision( $q->param('revision-note') );
        $app->add_return_arg( 'saved_snapshot' => 1 );
    }
    else {
        $app->add_return_arg( 'no_snapshot' => 1 );
    }

#    my $status = $q->param('status');
#    if ($status == 20 || $status == 40) {
#        _send_mail($app, $obj, $status);
#    }

############################
    my $entry = MT::Entry->load( $id );
    my $status = $q->param('status');
    $entry->status(40) if $status == 40;
    $entry->save;
############################

    $app->add_return_arg( 'id' => $obj->id ) if !$original->id;
    $app->call_return;
}

sub _gather_changed_cols {
    my ( $obj, $orig_obj, $app ) = @_;

    my @changed_cols;
    my $revisioned_cols = $obj->revisioned_columns;

    my %date_cols = map { $_ => 1 }
        @{ $obj->columns_of_type( 'datetime', 'timestamp' ) };

    foreach my $col (@$revisioned_cols) {
        next if $orig_obj && $obj->$col eq $orig_obj->$col;
        next
            if $orig_obj
                && exists $date_cols{$col}
                && $orig_obj->$col eq MT::Object::_db2ts( $obj->$col );

        push @changed_cols, $col;
    }

#    $obj->{changed_revisioned_cols} = \@changed_cols
#        if @changed_cols;

#    return 1 if @changed_cols;

    my $tag_changed = 0;

    my @objecttags = @{ $orig_obj->{__tags} };

    my @tag_names = split /\s*,\s*/,
            ( $app->param('tags') || '' );

    $tag_changed = 1 if scalar(@tag_names) != scalar(@objecttags);

    unless ($tag_changed) {
        if (@tag_names && @objecttags) {
            my @sort_tag_names = sort { $a <=> $b } @tag_names;
            my @sort_objecttags = sort { $a <=> $b } @objecttags;
            $tag_changed = 1 if join('', @sort_tag_names) ne join('', @sort_objecttags);
        }
    }
    push @changed_cols, 'tags' if $tag_changed;

    my $cat_changed = 0;

    my @category_list = @{ $orig_obj->{__cats} };

    my @category_ids;
    eval {
        @category_ids = split /\s*,\s*/,
            ( $app->param('category_ids') || '' );
        @category_ids = grep { $_ != -1 } @category_ids;
    };

    $cat_changed = 1 if scalar(@category_ids) != scalar(@category_list);

    unless ($cat_changed) {
        if (@category_ids && @category_list) {
            my @sort_category_ids = sort { $a <=> $b } @category_ids;
            my @sort_category_list = sort { $a <=> $b } @category_list;
            $cat_changed = 1 if join('', @sort_category_ids) ne join('', @sort_category_list);
        }
    }

    push @changed_cols, 'categories' if $cat_changed;

#    object asset
#    my $orig_object_assets = $obj->cache_property( 'orig_object_assets' );
#    my $object_assets = $obj->cache_property( 'object_assets' );
#    return 1 if !$orig_object_assets && !$object_assets;

#    my @orig_assets = sort { $a <=> $b } @$orig_object_assets;
#    my @assets = sort { $a <=> $b } @$object_assets;

#    $obj->cache_property( 'orig_object_assets', undef, undef );
#    return 1 if join('', @orig_assets) == join('', @assets);

#    my $changed_cols = $obj->{changed_revisioned_cols};
#    push @$changed_cols, 'assets';

#    return 1 if @changed_cols;

    my $meta = get_meta($orig_obj);
    while ( my ( $key, $val ) = each %$meta ) {
        if ( my $newval = $app->param( 'customfield_' . $key ) ) {
            if ( $val ne $newval ) {
                push @changed_cols, "field.$key";
            }
        } else {
            my $field = CustomFields::Field->load(
                {   $app->blog->id
                    ? ( blog_id => [ $app->blog->id, 0 ] )
                    : ( blog_id => $app->blog->id ),
                    basename => $key
                });
            if ( $field->type eq 'datetime' ) {
                if ( $field->options eq 'datetime' ) {
                    $newval = $app->param( 'd_customfield_' . $key ) . " " .
                              $app->param( 't_customfield_' . $key );
                } elsif ( $field->options eq 'date' ) {
                    $newval = $app->param( 'd_customfield_' . $key );
                    $val =~ s/(\d{4}-\d{2}-\d{2}) \d{2}:\d{2}:\d{2}/$1/;
                } elsif ( $field->options eq 'time' ) {
                    $newval = $app->param( 't_customfield_' . $key );
                    $val =~ s/\d{4}-\d{2}-\d{2} (\d{2}:\d{2}:\d{2})/$1/;
                }
                if ( $val ne $newval ) {
                    push @changed_cols, "field.$key";
                }
            } else {
                push @changed_cols, "field.$key" if $val;
            }
        }
    }

    $obj->{changed_revisioned_cols} = \@changed_cols
        if @changed_cols;

    1;
}

sub _set_customfield_objs {
    my ( $app, $obj ) = @_;
    return 1 unless $app->isa('MT::App');

    my $q = $app->param;
    my $blog_id = $app->param('blog_id') || 0;

    return 1 if !$q->param('customfield_beacon');
    require CustomFields::Field;

    my $meta = {};
    my %date_fields;
    foreach ( $q->param() ) {
        next if $_ eq 'customfield_beacon';
        if (m/^customfield_(.*?)$/) {
            my $field_name = $1;
            if (m/^customfield_(.+?)_cb_beacon$/) {
                $field_name = $1;
                $meta->{$field_name}
                    = defined( $q->param("customfield_$field_name") )
                    ? $q->param("customfield_$field_name")
                    : '0';
            }
            else {
                my $field = CustomFields::Field->load(
                    {   $blog_id
                        ? ( blog_id => [ $blog_id, 0 ] )
                        : ( blog_id => $blog_id ),
                        basename => $field_name
                    }
                ) or next;
                $meta->{$field_name} = $q->param("customfield_$field_name");
            }
        } elsif (m/^([td]_)customfield_(.*?)$/) {
            my ( $td, $field_name ) = ( $1, $2 );
            if ( $td ) {
                $date_fields{$field_name} = 1;
            }
        }

    }
    for my $field_name ( keys %date_fields ) {
        my $field = CustomFields::Field->load(
            {   $blog_id
                ? ( blog_id => [ $blog_id, 0 ] )
                : ( blog_id => $blog_id ),
                basename => $field_name
            }
        ) or next;
        if ( $field->options eq 'datetime' ) {
            $meta->{$field_name} = $q->param("d_customfield_$field_name") . " " .
                                   $q->param("t_customfield_$field_name");
        }
        elsif ( $field->options eq 'date' ) {
            if ( $q->param("d_customfield_$field_name")) {
                $meta->{$field_name} = $q->param("d_customfield_$field_name") . ' 00:00:00';
            } else {
                $meta->{$field_name} = '';
            }
        }
        elsif ( $field->options eq 'time' ) {
            if ( $q->param("t_customfield_$field_name")) {
                $meta->{$field_name} = '1970-01-01 ' . $q->param("t_customfield_$field_name");
            } else {
                $meta->{$field_name} = '';
            }
        }
    }
    _save_meta_wf( $obj, $meta ) if %$meta;

    1;
}

sub _save_meta_wf {
    my $plugin = MT->component("Commercial");
    my ( $obj, $meta ) = @_;
    return 0 unless $obj;
    my $obj_type
        = $obj->can('class_type') ? $obj->class_type : $obj->datasource;

    if ( $obj->has_meta ) {
        foreach my $mf ( keys %$meta ) {
            if ( $obj->has_meta( 'field.' . $mf ) ) {
                $obj->meta( 'field.' . $mf, $meta->{$mf} );
            }
        }
    }
    else {
        my $id = $obj->id;

        require MT::PluginData;
        my $meta_data = MT::PluginData->get_by_key(
            { plugin => 'CustomFields', key => "${obj_type}_${id}" } );
        $meta_data->data( { customfields => $meta } );
    }
    sync_assets($obj) unless $obj->isa('MT::Entry');

    return 1;
}

sub pack_revision_asset {
    my $obj = shift;
    my ($values) = @_;

    $values->{__rev_assets} = $obj->cache_property( 'object_assets' );
    $values;
}

sub unpack_revision_asset {
    my $obj = shift;
    my ($packed_obj) = @_;

    if ( my $rev_assets = delete $packed_obj->{__rev_assets} ) {
        $obj->clear_cache('assets');
        $obj->cache_property( 'object_assets', undef, \@$rev_assets );
    }
}

sub init_request {
    my $plugin = MT->component("PublishDraft");
    my $column = $plugin->get_config_value('column', 'system') || '';
    if ($column eq '') {
        $plugin->set_config_value('column', time(), 'system');
    }
}

##############################################
sub plugin_publish_future_posts {
    my $this = shift;

    require MT::Blog;
    require MT::Entry;
    require MT::Util;
    my $mt            = MT->instance;
    my $total_changed = 0;
    my @blogs         = MT::Blog->load(
        { class => '*' },
        {   join => MT::Entry->join_on(
                'blog_id',
                { status => 40, },
                { unique => 1 }
            )
        }
    );
    foreach my $blog (@blogs) {
        my @ts = MT::Util::offset_time_list( time, $blog );
        my $now = sprintf "%04d%02d%02d%02d%02d%02d", $ts[5] + 1900,
            $ts[4] + 1,
            @ts[ 3, 2, 1, 0 ];
        my $iter = MT::Entry->load_iter(
            {   blog_id => $blog->id,
                status  => 40,
                class   => '*'
            },
            {   'sort'    => 'authored_on',
                direction => 'descend'
            }
        );
        my @queue;
        while ( my $entry = $iter->() ) {
            push @queue, $entry->id if $entry->authored_on le $now;
        }

        my $changed = 0;
        my @results;
        my %rebuild_queue;
        my %ping_queue;
        foreach my $entry_id (@queue) {
            my $entry = MT::Entry->load($entry_id)
                or next;

##############
    my $revision = get_lastest_revision_for_future_post($mt, $entry->class, $entry_id);
    next if !$revision;

    my $obj = get_lastest_revision_object_for_future_post($mt, $entry->class, $entry_id, $revision);
    next if !$obj;
##############

            $obj->status( MT::Entry::RELEASE() );
            $obj->save
                or die $obj->errstr;

            my $col = 'max_revisions_' . $obj->datasource;
            if ( my $blog = $obj->blog ) {
                my $max = $blog->$col;
                $obj->handle_max_revisions($max);
            }
            my @changed_cols;
            push @changed_cols, 'status';
            $obj->{changed_revisioned_cols} = \@changed_cols;

            # for revision number hack
            my $current_revision = $obj->current_revision;
            $obj->current_revision( $current_revision + 2 );
            my $plugin = MT->component('PublishDraft');
            $revision = $obj->save_revision( $plugin->translate('by Schduled Task') );
            $obj->current_revision($revision - 1);
            $obj->save
                or die $obj->errstr;

            MT->run_callbacks( 'scheduled_post_published', $mt, $obj );

            $rebuild_queue{ $obj->id } = $obj;
            $ping_queue{ $obj->id }    = 1;
            my $n = $obj->next(1);
            $rebuild_queue{ $n->id } = $n if $n;
            my $p = $obj->previous(1);
            $rebuild_queue{ $p->id } = $p if $p;
            $changed++;
            $total_changed++;
        }
        if ($changed) {
            my %rebuilt_okay;
            my $rebuilt;
            eval {
                foreach my $id ( keys %rebuild_queue )
                {
                    my $entry = $rebuild_queue{$id};
                    $mt->rebuild_entry( Entry => $entry, Blog => $blog )
                        or die $mt->errstr;
                    $rebuilt_okay{$id} = 1;
                    if ( $ping_queue{$id} ) {
                        $mt->ping_and_save( Entry => $entry, Blog => $blog );
                    }
                    $rebuilt++;
                }
                $mt->rebuild_indexes( Blog => $blog )
                    or die $mt->errstr;
            };
            if ( my $err = $@ ) {

                # a fatal error occured while processing the rebuild
                # step. LOG the error and revert the entry/entries:
                require MT::Log;
                $mt->log(
                    {   message => $mt->translate(
                            "An error occurred while publishing scheduled entries: [_1]",
                            $err
                        ),
                        class    => "publish",
                        category => 'rebuild',
                        blog_id  => $blog->id,
                        level    => MT::Log::ERROR()
                    }
                );
                foreach my $id (@queue) {
                    next if exists $rebuilt_okay{$id};
                    my $e = $rebuild_queue{$id};
                    next unless $e;
                    $e->status( 40 );
                    $e->save or die $e->errstr;
                }
            }
        }
    }
    $total_changed > 0 ? 1 : 0;
}
##############################################

sub is_limit {
    return (rest_day() > 0) ? 0 : 1;
}

sub rest_day {
    my $plugin = MT->component("PublishDraft");
    my $column = $plugin->get_config_value('column', 'system') || '';
    my $duration = time() - $column;
    my $rest_day = 31 - int($duration / (24 * 3600));
    return ($rest_day > 0) ? $rest_day : 0;
}

1;
